package me.akito123321.valoStrats.rest.controller;

import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.http.HttpStatus;
import org.springframework.http.ResponseEntity;
import org.springframework.security.core.Authentication;
import org.springframework.security.core.context.SecurityContextHolder;
import org.springframework.web.bind.annotation.DeleteMapping;
import org.springframework.web.bind.annotation.GetMapping;
import org.springframework.web.bind.annotation.PathVariable;
import org.springframework.web.bind.annotation.PostMapping;
import org.springframework.web.bind.annotation.PutMapping;
import org.springframework.web.bind.annotation.RequestBody;
import org.springframework.web.bind.annotation.RequestMapping;
import org.springframework.web.bind.annotation.RestController;

import me.akito123321.valoStrats.rest.requests.GroupRequest;
import me.akito123321.valoStrats.rest.requests.StratRequest;
import me.akito123321.valoStrats.rest.requests.StratStateRequest;
import me.akito123321.valoStrats.rest.services.GroupService;
import me.akito123321.valoStrats.rest.services.StratService;
import me.akito123321.valoStrats.rest.services.StratStateService;
import me.akito123321.valoStrats.rest.util.JDBCUserDetailsService;
import me.akito123321.valoStrats.schemas.Group;
import me.akito123321.valoStrats.schemas.Strat;
import me.akito123321.valoStrats.schemas.StratState;
import me.akito123321.valoStrats.schemas.StratsUser;

@RestController
@RequestMapping("/api/group")
public class GroupRestAPI {
	
	@Autowired
	private StratStateService stratStateService;

	@Autowired
	private JDBCUserDetailsService userService;

	@Autowired
	private GroupService groupService;
	
	@Autowired
	private StratService stratService;

	@PostMapping()
	public ResponseEntity<Group> createGroup(@RequestBody GroupRequest request) {
		StratsUser user = getUser();
		Group group = new Group(request.name(), user);
		groupService.saveGroup(group);
		return ResponseEntity.ok(group);
	}

	@GetMapping()
	public ResponseEntity<List<Group>> getGroups() {
		StratsUser user = getUser();
		List<Group> groups = user.getGroups();
		return ResponseEntity.ok(groups);
	}

	@DeleteMapping("/{id}")
	public ResponseEntity<Void> removeGroup(@PathVariable String id) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getOwner().getUsername().equals(user.getUsername())) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		groupService.removeGroup(id);

		return ResponseEntity.ok(null);
	}

	@PutMapping("/{id}")
	public ResponseEntity<Group> updateGroup(@PathVariable String id, @RequestBody GroupRequest request) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getOwner().getUsername().equals(user.getUsername())) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		group.setName(request.name());

		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}

	@GetMapping("/{id}")
	public ResponseEntity<Group> getGroup(@PathVariable String id) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		return ResponseEntity.ok(group);
	}

	@PostMapping("/{id}/member")
	public ResponseEntity<Group> addUserToGroup(@PathVariable String id, @RequestBody String userId) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getOwner().equals(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		group.getMembers().add(userService.loadUserByUsername(userId));

		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}

	@DeleteMapping("/{id}/member/{userId}")
	public ResponseEntity<Group> removeUserFromGroup(@PathVariable String id, @PathVariable String userId) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getOwner().getUsername().equals(user.getUsername()) || !user.getUsername().equals(userId)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		group.getMembers().remove(userService.loadUserByUsername(userId));

		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}

	@PostMapping("/{id}/strat")
	public ResponseEntity<Group> addStrat(@PathVariable String id, @RequestBody StratRequest stratRequest) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		Strat strat = new Strat(stratRequest.title(), stratRequest.description(), group, stratRequest.stratType());
		
		if(stratRequest.attempts() != null) {
			strat.setAttempts(stratRequest.attempts());
		}
		if(stratRequest.success() != null) {
			strat.setSuccess(stratRequest.success());
		}

		stratService.saveStrat(strat);
		
		group.getStrats().add(strat);
		
		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}

	@DeleteMapping("/{id}/strat/{stratId}")
	public ResponseEntity<Group> removeStrat(@PathVariable String id, @PathVariable String stratId) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		Strat strat = stratService.getStratById(stratId);
		if (strat == null) {
			return ResponseEntity.notFound().build();
		}
		
		stratService.removeStrat(strat);

		group.getStrats().remove(strat);

		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}
	
	@PutMapping("/{id}/strat/{stratId}")
	public ResponseEntity<Group> editStrat(@PathVariable String id, @PathVariable String stratId, @RequestBody StratRequest newStrat) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		if (group == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}

		Strat strat = stratService.getStratById(stratId);
		if (strat == null){
			return ResponseEntity.notFound().build();
		}if (!group.getStrats().contains(strat)) {
			return ResponseEntity.badRequest().build();
		}
		
		strat.setDescription(newStrat.description());
		strat.setTitle(newStrat.title());
		strat.setStratStates(newStrat.stratStates());
		strat.setStratType(newStrat.stratType());
		if(newStrat.attempts() != null) {
			strat.setAttempts(newStrat.attempts());
		}
		if(newStrat.success() != null) {
			strat.setSuccess(newStrat.success());
		}
		
		stratService.saveStrat(strat);
		
		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}
	
	@PostMapping("/{id}/strat/{stratId}/strat-state")
	public ResponseEntity<Group> addStratState(@PathVariable String id, @PathVariable String stratId , @RequestBody StratStateRequest stratStateRequest) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		Strat strat = stratService.getStratById(stratId);
		
		if (group == null | strat == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}
		if(!group.getStrats().contains(strat)) {
			return ResponseEntity.badRequest().build();
		}
		
		StratState stratState = new StratState(stratStateRequest.description(), strat);
		
		stratStateService.saveStratState(stratState);

		strat.getStratStates().add(stratState);
		
		stratService.saveStrat(strat);
		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}

	@DeleteMapping("/{id}/strat/{stratId}/strat-state/{stratStateId}")
	public ResponseEntity<Group> removeStratState(@PathVariable String id, @PathVariable String stratId, @PathVariable String stratStateId) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		Strat strat = stratService.getStratById(stratId);
		StratState stratState = stratStateService.getStratStateById(stratStateId);
		
		if (group == null | strat == null | stratState == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}
		if(!group.getStrats().contains(strat) | !strat.getStratStates().contains(stratState)) {
			return ResponseEntity.badRequest().build();
		}
		
		stratStateService.removeStratState(stratState);
		
		strat.getStratStates().remove(stratState);
		
		stratService.saveStrat(strat);
		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}
	
	@PutMapping("/{id}/strat/{stratId}/strat-state/{stratStateId}")
	public ResponseEntity<Group> editStratState(@PathVariable String id, @PathVariable String stratId, @PathVariable String stratStateId, @RequestBody StratStateRequest newStratState) {
		StratsUser user = getUser();

		Group group = groupService.getGroupById(id);
		Strat strat = stratService.getStratById(stratId);
		StratState stratState = stratStateService.getStratStateById(stratStateId);
		
		if (group == null | strat == null | stratState == null) {
			return ResponseEntity.notFound().build();
		}
		if (!group.getMembers().contains(user)) {
			return ResponseEntity.status(HttpStatus.FORBIDDEN).build();
		}
		if(!group.getStrats().contains(strat) | !strat.getStratStates().contains(stratState)) {
			return ResponseEntity.badRequest().build();
		}
		
		stratState.setDescription(newStratState.description());
		stratState.setImage(newStratState.image());
		
		stratStateService.saveStratState(stratState);
		stratService.saveStrat(strat);
		groupService.saveGroup(group);

		return ResponseEntity.ok(group);
	}
	

	private StratsUser getUser() {
		Authentication auth = SecurityContextHolder.getContext().getAuthentication();
		StratsUser user = (StratsUser) auth.getPrincipal();
		return user;
	}

}
